@@ -29,18 +29,46 @@ from pwndbg.aglib import load_aglib
 from pwndbg.dbg import selection
 from pwndbg.gdblib import gdb_version
 from pwndbg.gdblib import load_gdblib
+from pwndbg.lib.arch import PWNDBG_SUPPORTED_ARCHITECTURES_TYPE
 from pwndbg.lib.memory import PAGE_MASK
 from pwndbg.lib.memory import PAGE_SIZE
 
 T = TypeVar("T")
 
 
+# List of supported architectures that GDB recognizes
+# These strings to converted to the Pwndbg-specific name for the architecture
+gdb_architecture_name_fixup_list = (
+    "x86-64",
+    "i386",
+    "i8086",
+    "aarch64",
+    "mips",
+    "rs6000",
+    "powerpc",
+    "sparc",
+    "arm",
+    "iwmmxt",
+    "iwmmxt2",
+    "xscale",
+    "riscv:rv32",
+    "riscv:rv64",
+    "riscv",
+    "loongarch64",
+)
+
+
 class GDBArch(pwndbg.dbg_mod.Arch):
     _endian: Literal["little", "big"]
-    _name: str
+    _name: PWNDBG_SUPPORTED_ARCHITECTURES_TYPE
     _ptrsize: int
 
-    def __init__(self, endian: Literal["little", "big"], name: str, ptrsize: int):
+    def __init__(
+        self,
+        endian: Literal["little", "big"],
+        name: PWNDBG_SUPPORTED_ARCHITECTURES_TYPE,
+        ptrsize: int,
+    ):
         self._endian = endian
         self._name = name
         self._ptrsize = ptrsize
@@ -52,7 +80,7 @@ class GDBArch(pwndbg.dbg_mod.Arch):
 
     @override
     @property
-    def name(self) -> str:
+    def name(self) -> PWNDBG_SUPPORTED_ARCHITECTURES_TYPE:
         return self._name
 
     @override
@@ -136,7 +164,7 @@ class GDBFrame(pwndbg.dbg_mod.Frame):
 
     @override
     def reg_write(self, name: str, val: int) -> bool:
-        if name not in pwndbg.aglib.regs.all:
+        if name not in pwndbg.aglib.regs:
             return False
 
         with selection(self.inner, lambda: gdb.selected_frame(), lambda f: f.select()):
@@ -402,46 +430,74 @@ class GDBProcess(pwndbg.dbg_mod.Process):
         pages.sort()
         return GDBMemoryMap(qemu, pages)
 
+    def _is_memory_readable(self, addr: int) -> bool:
+        try:
+            gdb.selected_inferior().read_memory(addr, 1)
+            return True
+        except gdb.error:
+            return False
+
+    def _find_memory_last_readable(self, start: int, count: int) -> int:
+        end = start + count
+        result = -1
+
+        if not self._is_memory_readable(start):
+            return result
+
+        while start <= end:
+            mid = (start + end + 1) // 2
+            if self._is_memory_readable(mid):
+                result = mid
+                start = mid + 1
+            else:
+                end = mid - 1
+
+        return result
+
     @override
     def read_memory(self, address: int, size: int, partial: bool = False) -> bytearray:
-        result = b""
         count = max(int(size), 0)
         addr = address
 
         try:
             result = gdb.selected_inferior().read_memory(addr, count)
+            return bytearray(result)
         except gdb.error as e:
             if not partial:
                 raise pwndbg.dbg_mod.Error(e)
 
-            message = str(e)
+            if not pwndbg.aglib.remote.is_remote():
+                message = str(e)
+                match = re.search(r"Memory at address (\w+) unavailable\.", message)
+                if match:
+                    stop_addr = int(match.group(1), 0)
+                else:
+                    stop_addr = int(message.split()[-1], 0)
 
-            stop_addr = addr
-            match = re.search(r"Memory at address (\w+) unavailable\.", message)
-            if match:
-                stop_addr = int(match.group(1), 0)
+                # Handle case of memory read that wraps around the memory space back to 0, where high memory was readable but memory at 0 was not.
+                # Example: 2-byte read at 0xFFFF_FFFF in a 32-bit address space.
+                # GDB returns error: "Cannot access memory at address 0x0"
+                if stop_addr == 0 and stop_addr < addr:
+                    # We could read from the top-portion of memory, but not after wrapping around
+                    # Because we are doing a partial read, read until the max address
+                    return self.read_memory(addr, pwndbg.aglib.arch.ptrmask - addr + 1)
+
+                if stop_addr > addr:
+                    return self.read_memory(addr, stop_addr - addr)
             else:
-                stop_addr = int(message.split()[-1], 0)
+                # Handle the case of remote debugging, where GDB's remote protocol
+                # returns the start address as the failed read address instead of the stop address.
+                # This is a limitation in how GDB handles the remote protocol, and while it could
+                # be fixed, it currently behaves this way.
+                #
+                # To work around this, we perform a binary search in the `_find_memory_last_readable` method
+                # to find the correct stop address that avoids the failure.
+                #
+                # For local debugging, this issue does not occur, and we proceed with the normal flow.
+                if (stop_addr := self._find_memory_last_readable(addr, count)) > 0:
+                    return self.read_memory(addr, stop_addr - addr + 1)
 
-            if stop_addr != addr:
-                return self.read_memory(addr, stop_addr - addr)
-
-            # QEMU will return the start address as the failed
-            # read address.  Try moving back a few pages at a time.
-            stop_addr = addr + count
-
-            # Move the stop address down to the previous page boundary
-            stop_addr &= PAGE_MASK
-            while stop_addr > addr:
-                result = self.read_memory(addr, stop_addr - addr)
-
-                if result:
-                    return bytearray(result)
-
-                # Move down by another page
-                stop_addr -= PAGE_SIZE
-
-        return bytearray(result)
+            raise pwndbg.dbg_mod.Error(e)
 
     @override
     def write_memory(self, address: int, data: bytearray, partial: bool = False) -> int:
@@ -570,10 +626,14 @@ class GDBProcess(pwndbg.dbg_mod.Process):
 
         if pwndbg.aglib.file.is_vfile_qemu_user_bug():
             with open(local_path, "wb") as fp:
-                for data in pwndbg.aglib.file.vfile_readfile(remote_path):
-                    fp.write(data)
-            return
-
+                try:
+                    for data in pwndbg.aglib.file.vfile_readfile(remote_path):
+                        fp.write(data)
+                    return
+                except OSError as e:
+                    raise pwndbg.dbg_mod.Error(
+                        "Could not download remote file %r:\nError: %s" % (remote_path, str(e))
+                    )
         try:
             error = gdb.execute(f'remote get "{remote_path}" "{local_path}"', to_string=True)
         except gdb.error as e:
@@ -678,8 +738,10 @@ class GDBProcess(pwndbg.dbg_mod.Process):
             arch = gdb.execute("show architecture", to_string=True).strip()
             not_exactly_arch = True
 
+        arch = arch.lower()
+
         # Below, we fix the fetched architecture
-        for match in pwndbg.aglib.arch_mod.ARCHS:
+        for match in gdb_architecture_name_fixup_list:
             if match in arch:
                 # Distinguish between Cortex-M and other ARM
                 # When GDB detects correctly Cortex-M processes, it will label them with `arm*-m`, such as armv7e-m
@@ -695,12 +757,17 @@ class GDBProcess(pwndbg.dbg_mod.Process):
                 elif match == "riscv":
                     # If GDB doesn't detect the width, it will just say `riscv`.
                     match = "rv64"
-                return GDBArch(endian, match, ptrsize)
+                elif match == "iwmmxt" or match == "iwmmxt2" or match == "xscale":
+                    match = "arm"
+                elif match == "rs6000":
+                    # The RS/6000 architecture is compatible with the PowerPC common
+                    match = "powerpc"
+                return GDBArch(endian, match, ptrsize)  # type: ignore[arg-type]
 
         if not_exactly_arch:
             raise RuntimeError(f"Could not deduce architecture from: {arch}")
 
-        return GDBArch(endian, arch, ptrsize)
+        return GDBArch(endian, arch, ptrsize)  # type: ignore[arg-type]
 
     @override
     def break_at(
@@ -1227,6 +1294,10 @@ def _gdb_event_class_from_event_type(ty: pwndbg.dbg_mod.EventType) -> Any:
 class GDB(pwndbg.dbg_mod.Debugger):
     @override
     def setup(self):
+        import pwnlib.update
+
+        pwnlib.update.disabled = True
+
         from pwndbg.commands import load_commands
 
         load_gdblib()
@@ -1502,6 +1573,40 @@ class GDB(pwndbg.dbg_mod.Debugger):
     def supports_breakpoint_creation_during_stop_handler(self) -> bool:
         return False
 
+    @override
+    def breakpoint_locations(self) -> List[pwndbg.dbg_mod.BreakpointLocation]:
+        bps = gdb.breakpoints()
+        locations: List[pwndbg.dbg_mod.BreakpointLocation] = []
+        for bp in bps:
+            if (
+                bp.is_valid()
+                and bp.enabled
+                and bp.type in (gdb.BP_BREAKPOINT, gdb.BP_HARDWARE_BREAKPOINT)
+                and bp.visible
+            ):
+                # GDB 13.1+
+                if hasattr(bp, "locations"):
+                    for location in bp.locations:
+                        locations.append(pwndbg.dbg_mod.BreakpointLocation(location.address))
+                else:
+                    # Num     Type           Disp Enb Address            What
+                    # 1       breakpoint     keep y   0x00007ffff7e90840 in __GI___libc_read at ../sysdeps/unix/sysv/linux/read.c:26
+                    bp_locations = gdb.execute(
+                        f"info breakpoint {bp.number}", to_string=True
+                    ).split("\n")
+                    for line in bp_locations:
+                        try:
+                            address = int(line.split()[4], 16)
+                            locations.append(pwndbg.dbg_mod.BreakpointLocation(address))
+                        except (IndexError, ValueError):
+                            # Ignore lines that don't have an address.
+                            pass
+        return locations
+
+    @override
+    def name(self) -> pwndbg.dbg_mod.DebuggerType:
+        return pwndbg.dbg_mod.DebuggerType.GDB
+
     @override
     def x86_disassembly_flavor(self) -> Literal["att", "intel"]:
         try:
@@ -1551,6 +1656,13 @@ class GDB(pwndbg.dbg_mod.Debugger):
             width if width is None else int(width),
         )
 
+    @override
+    @property
+    def pre_ctx_lines(self) -> int:
+        # GDB most often prints one line
+        # as a "0x000055555556fa8f in main ()"-type message
+        return 1
+
     @override
     def set_python_diagnostics(self, enabled: bool) -> None:
         if enabled:
